#!/usr/bin/env python3
import re
import gzip
import random
import subprocess
from urllib.parse import urlparse
from urllib.request import Request, urlopen

from .parsers import StopParsing


USERAGENTS = (
    "Dillo/3.0.5",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/90.0.4430.212 Safari/537.36",
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.63 Safari/537.36 Edg/93.0.961.38",
    "Opera/9.80 (Windows NT 6.1; WOW64) Presto/2.12.388 Version/12.18",
    "Opera/9.80 (Linux armv7l) Presto/2.12.407 Version/12.51 , D50u-D1-UHD/V1.5.16-UHD (Vizio, D50u-D1, Wireless)",
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.130 Safari/537.36 OPR/66.0.3515.72",
    "Links (2.22; Linux X86_64; GNU C; text)",
    "Mozilla/5.0 (Windows NT 6.3; Win64; x64; rv:73.0) Gecko/20100101 Firefox/73.0",
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:58.0) Gecko/20100101 Firefox/58.0",
    "Mozilla/5.0 (X11; Linux x86_64; rv:38.0) Gecko/20100101 Firefox/38.0 Iceweasel/38.4.0",
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.111 Safari/537.36 Edge/86.0.622.51"
)
HEADERS = {
    "User-Agent": random.choice(USERAGENTS),
    "Accept": "text/html, application/xhtml+xml, image/jxr, */*",
    "Accept-Encoding": "gzip",
    "Accept-Language": "ru,en;q=0.9,en-GB;q=0.8,en-US;q=0.7"
}


def isallow(url):
    match = re.search(
        r"(https://www\.(?:imdb|kinopoisk)\.(?:ru|com)/(?:film|title)/(?:tt)*(?:\d+)/)",
        url)
    if match:
        return True
    return False


def getd(data):
    if data[:2] == b"\x1f\x8b":
        data = gzip.decompress(data)
    return data


def getr(url):
    try:
        r = urlopen(Request(url, headers=HEADERS))
    except:
        return None
    return r


def geth(url):
    data = b""
    ua = HEADERS["User-Agent"]
    if "kinopoisk" in url:
        r = subprocess.run(f'wget -qO- -U "{ua}" {url}', shell=True,
            capture_output=True)
        if r.returncode == 0:
            data = r.stdout
    else:
        r = getr(url)
        if r:
            data = r.read()
    if data:
        data = getd(data)
    return data.decode("utf8", errors="ignore")


def parser_run_and_stop(parser, data):
    try:
        parser.feed(data)
    except StopParsing:
        pass
    finally:
        parser.reset()
        parser.close()




#!/usr/bin/env python3
import os
import time
from urllib.parse import urljoin, quote

from .parsers import ImdbSearchParser, KinopoiskSearchParser, \
    KinopoiskParser, ImdbParser, KinopoiskSeriesParser, ImdbSeriesParser
from .utils import isallow, getd, getr, geth, parser_run_and_stop


class Film:
    def __init__(self, **kargs):
        for k, v in kargs.items():
            if v is not None:
                if isinstance(v, str) and v == "":
                    continue
                self.__dict__[k] = v
    
    def __getattr__(self, name):
        if name in self.__dict__:
            return self.__dict__[name]
        else:
            return None
    
    def __setattr__(self, name, value):
        self.__dict__[name] = value
    
    def __str__(self):
        kargs = []
        title = ""
        
        for k, v in self.__dict__.items():
            if v is not None:
                if k == "title":
                    title = v
                else:
                    kargs.append(str(v))
        
        return f"{title} ({', '.join(kargs)})"
    
    def __repr__(self):
        kargs = []
        
        for k, v in self.__dict__.items():
            if v is not None:
                if isinstance(v, str) and v != "":
                    kargs.append(f"{k}='{v}'")
                else:
                    kargs.append(f"{k}={v}")
        
        return f"{self.__class__.__name__}({','.join(kargs)})"


class Series(Film):
    def __init__(self, **kargs):
        super().__init__(**kargs)
        self.titles = {}
        self._s = {}
    
    def __str__(self):
        kargs = []
        
        if self._s:
            for k, v in self._s.items():
                kargs.append(f"{k}: {v}")
            
            return ", ".join(kargs)
        else:
            title = ""
            
            for k, v in self.__dict__.items():
                if v is not None and k not in ("titles", "_s"):
                    if k == "title":
                        title = v
                    else:
                        kargs.append(str(v))
            
            return f"{title} ({', '.join(kargs)})"
    
    def __repr__(self):
        kargs = []
        
        if self._s:
            for k, v in self._s.items():
                kargs.append(f"{k}: {v}")
            
            return repr(", ".join(kargs))
        else:
            for k, v in self.__dict__.items():
                if not v and k == "titles":
                    continue
                elif v is not None and k not in ("_s"):
                    if isinstance(v, str) and v != "":
                        kargs.append(f"{k}='{v}'")
                    else:
                        kargs.append(f"{k}={v}")
            
            return f"{self.__class__.__name__}({','.join(kargs)})"
    
    def add_season(self, season, episodes):
        if isinstance(episodes, dict) and isinstance(season, int):
            ep = {}
            
            for k, v in episodes.items():
                if isinstance(k, int) and isinstance(v, str):
                    ep[k] = v
            
            if ep:
                self.titles[season] = ep
    
    def amount_seasons(self):
        return len(self.titles)
    
    def amount_episodes(self, num=None):
        amount = 0
        
        if num is None:
            for k, v in self.titles.items():
                amount += len(v)
        else:
            try:
                num = int(num)
            except (TypeError, ValueError):
                amount = 0
            
            episodes = self.titles.get(num)
            
            if episodes is None:
                amount = 0
            else:
                amount = len(episodes)
        
        return amount
    
    def s(self, num):
        if not isinstance(num, int):
            num = int(num)
        
        if not self.titles:
            self.fetch_titles()
        
            if not self.titles:
                return None
        
        if min(i for i in self.titles) <= num <= self.amount_seasons():
            self._s = self.titles[num]
            return self
        else:
            return None
    
    def ep(self, num):
        if not self._s:
            return None
        
        if not isinstance(num, int):
            num = int(num)
        
        ep = self._s.get(num)
        self._s = {}
        
        return ep
    
    def fetch_titles(self):
        try:
            url = self.url
        except KeyError:
            return False
        
        if not url.endswith("/"):
            url += "/"
            self.url = url
        
        if not isallow(url):
            return False
        
        if "kinopoisk" in url:
            parser = KinopoiskSeriesParser
        else:
            parser = ImdbSeriesParser
        
        html = geth(urljoin(url, "episodes/"))
        
        p = parser()
        parser_run_and_stop(p, html)
        
        self.titles = p.titles
        
        if "imdb" in url:
            uj = urljoin(url, "episodes?season=")
            
            try:
                season_last = int(p._season)
            except ValueError:
                return True if self.titles else False
            
            for i in range(1, season_last):
                time.sleep(1)
                
                html = geth(uj + str(i))
                if not html:
                    continue
                
                ps = parser()
                parser_run_and_stop(ps, html)
                
                self.titles.update(ps.titles)
        
        return True if self.titles else False


def new(url):
    if not url.endswith("/"):
        url += "/"
    
    if not isallow(url):
        return None
    
    if "kinopoisk" in url:
        p = KinopoiskParser()
    else:
        p = ImdbParser()
    
    html = geth(url)
    
    parser_run_and_stop(p, html)
    
    keys = ("title", "alternate", "year", "time", "age", "url")
    values = (p.title, p.alternate, p.year, p.time, p.age, url)
    
    args = {k: v for k, v in zip(keys, values)}
    
    if p.isfilm:
        return Film(**args)
    else:
        return Series(**args)


def search(word):
    if len(word) < 2:
        return []
    key = quote(word.lower())
    result = []
    for url, parser in (("https://www.imdb.com/find?q=", ImdbSearchParser),
      ("https://www.kinopoisk.ru/index.php?kp_query=", KinopoiskSearchParser)):
        r = getr(url + key)
        if not r:
            continue
        data = getd(r.read())
        html = data.decode("utf8", errors="ignore")
        if "kinopoisk" in url and ("film" in r.url or "series" in r.url):
                p = KinopoiskParser()
                parser_run_and_stop(p, html)
                name = f"{p.title} / {p.alternate} ({p.year})"
                found = [(name, r.url)]
        else:
            p = parser()
            parser_run_and_stop(p, html)
            found = [(i, urljoin(url, j)) for i, j in p.found]
        result += found
    return result



#!/usr/bin/env python3
import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="imkin",
    version="0.2.4",
    author="shllw",
    author_email="shllw@yahoo.com",
    description="Lightweight a movie and TV series data parser from imdb.com and kinopoisk.ru",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords="parser imdb kinopoisk",
    url="https://bitbucket.org/shllw/imkin/",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Operating System :: POSIX :: Linux",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
)



#!/usr/bin/env python3
import json
import html
from html.parser import HTMLParser


class StopParsing(Exception): pass


class BaseParser(HTMLParser):
    def __init__(self):
        super().__init__()
        self.title = ""
        self.alternate = None
        self.year = None
        self.time = None
        self.age = None
        self.isfilm = True


class ImdbParser(BaseParser):
    def __init__(self):
        super().__init__()
        self._script = False
        self._span = False
        self._spanRuntime = False
        self._spanEnd = False
        self._tagAfterSpan = ""
    
    def handle_starttag(self, tag, attrs):
        if tag == "script":
            for n, v in attrs:
                if n == "type" and v == "application/ld+json":
                    self._script = True
        elif tag == "span":
            for n, v in attrs:
                if n == "class" and "label" in v:
                    self._span = True
        elif self._spanEnd:
            self._tagAfterSpan = tag
    
    def handle_data(self, data):
        if self._script:
            j = json.loads(data.strip())
            
            self.title = html.unescape(j["name"])
            
            alternate = j.get("alternateName", "")
            if alternate:
                self.alternate = alternate
            
            year = j.get("datePublished", "")
            if year:
                try:
                    self.year = int(year.split("-")[0])
                except (ValueError, TypeError):
                    pass
            
            time = j.get("duration", "")
            if time:
                self.time = self._calc_time(time)
            else:
                self.time = ""
            
            age = j.get("contentRating", "")
            if age:
                self.age = age
            
            self.isfilm = True if j["@type"] == "Movie" else False
        elif self._span and data == "Runtime":
            self._spanRuntime = True
        elif self._tagAfterSpan:
            self.time += data

    def handle_endtag(self, tag):
        if tag == "script" and self._script:
            self._script = False
            if self.time:
                raise StopParsing()
        elif tag == "span" and self._span and self._spanRuntime:
            self._span = False
            self._spanRuntime = False
            self._spanEnd = True
        elif tag == self._tagAfterSpan:
            self.time = self._calc_time(self.time)
            raise StopParsing()
            
    
    def _calc_time(self, s):
        if "hour" in s:
            time_split = s.strip().lower().split("hour")
        else:
            time_split = s.strip().lower().split("h")
        
        if len(time_split) == 1:
             time = "".join(i for i in time_split[0] if i.isdigit())
             try:
                return int(time)
             except (ValueError, TypeError):
                pass
        else:
            hours = "".join(i for i in time_split[0] if i.isdigit())
            minutes = "".join(i for i in time_split[1] if i.isdigit())
            
            if not minutes:
                minutes = 0
            try:
                return int(hours) * 60 + int(minutes)
            except (ValueError, TypeError):
                pass
        
        return None


class KinopoiskParser(BaseParser):
    def __init__(self):
        super().__init__()
        self._script = False
        self._a = False
        self._span = False
    
    def handle_starttag(self, tag, attrs):
        if tag == "script":
            for n, v in attrs:
                if n == "type" and v == "application/ld+json":
                    self._script = True
        elif tag == "a":
            for n, v in attrs:
                if not self.age and n == "class" and "restrictionLink" in v:
                    self._a = True
        elif tag == "span" and self._a:
            self._span = True
    
    def handle_data(self, data):
        if self._script:
            j = json.loads(data.strip())
            
            self.title = j["name"]
            
            alternate = j.get("alternateName", "")
            if alternate:
                self.alternate = html.unescape(alternate)
            
            try:
                self.year = int(j["datePublished"])
            except (ValueError, TypeError):
                pass
            
            try:
                self.time = int(j["timeRequired"])
            except (ValueError, TypeError):
                pass
            
            self.isfilm = True if j["@type"] == "Movie" else False
            self._script = False
        elif self._span:
            if "+" in data:
                self.age = data.strip()

    def handle_endtag(self, tag):
        if self._span:
            raise StopParsing()


class BaseSeriesParser(HTMLParser):
    def __init__(self):
        super().__init__()
        self.titles = {}
        self._season = None
        self._episode = None
    
    def _update_titles(self, season, episode, title):
        if not self.titles.get(season):
            self.titles[season] = {}
        e = self.titles[season]
        if not e.get(episode):
            self.titles[season].update({episode: title})
        else:
            t = e.get(episode)
            t += "/" + title
            self.titles[season].update({episode: t})


class ImdbSeriesParser(BaseSeriesParser):
    def __init__(self):
        super().__init__()
        self._option = False
        self._meta_episodes = 0
        self._div_eplist = False
        self._div_list_item = False
        self._meta = False
        self._strong = False
    
    def handle_starttag(self, tag, attrs):
        if tag == "option" and not self._option:
            for n, v in attrs:
                if n == "selected" and v == "selected":
                    self._option = True
                    continue
                if n == "value" and self._option:
                    self._season = int(v)
                    return
        if tag == "div":
            for n, v in attrs:
                if n == "class" and "eplist" in v:
                    self._div_eplist = True
                    return
                if n == "class" and "list_item" in v and self._div_eplist:
                    self._div_list_item = True
                    return
        if tag == "meta":
            ep_num = False
            q = False
            for n, v in attrs:
                if n == "itemprop" and v == "episodeNumber" and \
                   self._div_list_item:
                    ep_num = True
                if n == "itemprop" and v == "numberofEpisodes" and \
                   self._meta_episodes == 0:
                    q = True
            for n, v in attrs:
                if n == "content" and q:
                    self._meta_episodes = int(v)
                    q = False
                    return
                if n == "content" and ep_num:
                    self._episode = int(v)
                    self._meta = True
                    ep_num = False
                    return
        if tag == "strong" and self._meta:
            self._strong = True
            return
    
    def handle_data(self, data):
        if self._strong:
            name = data.strip()
            if name:
                self._update_titles(self._season, self._episode, name)
    
    def handle_endtag(self, tag):
        if self._strong:
            self._strong = False
            self._meta = False
            self._div_list_item = False
            if len(self.titles[self._season]) == self._meta_episodes:
                raise StopParsing()


class KinopoiskSeriesParser(BaseSeriesParser):
    def __init__(self):
        super().__init__()
        self._div_season_item = False
        self._a = False
        self._a_s = False
        self._table = False
        self._td = False
        self._span = False
        self._b = False
        self._span_original = False
        self._seasons = []
    
    def handle_starttag(self, tag, attrs):
        if tag == "div" and not self._div_season_item:
            for n, v in attrs:
                if n == "class" and v == "season_item":
                    self._div_season_item = True
                    return
        if tag == "a" and self._div_season_item:
            for n, v in attrs:
                if n == "href" and v.startswith("#"):
                    self._a = True
                    season = v[2:]
                    self._seasons.append(season)
                    self._a = False
                    return
        if tag == "a":
            for n, v in attrs:
                if n == "name" and v[1:] in self._seasons:
                    s = v[1:]
                    self._seasons.remove(s)
                    self._season = int(s)
                    self._a_s = True
                    return
        if tag == "table" and self._a_s:
            self._table = True
            return
        if tag == "td" and self._table:
            self._td = True
            return
        if tag == "span" and self._td:
            if not self._span:
                self._span = True
                return
            else:
                for n, v in attrs:
                    if n == "class" and v == "episodesOriginalName":
                        self._span_original = True
                        return
        if tag == "b" and self._span:
            self._b = True
            return
    
    def handle_data(self, data):
        if self._div_season_item:
            self._div_season_item = False
        elif self._b or self._span_original:
            name = data.strip()
            if name:
                self._update_titles(self._season, self._episode, name)
        elif self._span and not self._b:
            episode = data.strip()
            if "Эпизод" in episode:
                ep = episode.split()[-1]
                self._episode = int(ep)
    
    def handle_endtag(self, tag):
        if tag == "b" and self._b:
            self._b = False
        if tag == "span" and self._span_original:
            self._td = False
            self._span = False
            self._span_original = False
        if tag == "table" and self._table:
            self._table = False
            self._a_s = False
            if not self._seasons:
                raise StopParsing()


class BaseSearch(HTMLParser):
    def __init__(self):
        super().__init__()
        self.found = []
        self._name = ""
        self._url = ""


class ImdbSearchParser(BaseSearch):
    def __init__(self):
        super().__init__()
        self._table = False
        self._td = False
        self._a = False
    
    def handle_starttag(self, tag, attrs):
        if tag == "table" and not self._table:
            for n, v in attrs:
                if n == "class" and v == "findList":
                    self._table = True
        elif tag == "td" and self._table:
            for n, v in attrs:
                if n == "class" and v == "result_text":
                    self._td = True
        elif tag == "a" and self._td:
            for n, v in attrs:
                if n == "href":
                    self._url = v.split("?")[0]
                    self._a = True
    
    def handle_data(self, data):
        data = data.strip()
        if data:
            if self._a:
                self._name = data
            elif self._td:
                self._name += " " + data
    
    def handle_endtag(self, tag):
        if tag == "a" and self._a:
            self._a = False
        elif tag == "td" and self._td:
            if self._name and self._url:
                self.found.append((self._name, self._url))
            self._name = ""
            self._url = ""
            self._td = False
        elif tag == "table" and self._table:
            raise StopParsing()


class KinopoiskSearchParser(BaseSearch):
    def __init__(self):
        super().__init__()
        self._p = False
        self._a = False
    
    def handle_starttag(self, tag, attrs):
        if tag == "p":
            for n, v in attrs:
                if n == "class" and v == "name":
                    self._p = True
        elif tag == "a" and self._p:
            for n, v in attrs:
                if n == "data-url" and "film" in v:
                    self._url = v + "/"
                    self._a = True
        elif tag == "div":
            for n, v in attrs:
                if n == "class" and v == "search_gray":
                    raise StopParsing()
    
    def handle_data(self, data):
        data = data.strip()
        if data:
            if self._a:
                self._name = data
            elif self._p:
                self._name += " (" + data + ")"
    
    def handle_endtag(self, tag):
        if tag == "a" and self._a:
            self._a = False
        elif tag == "p" and self._p:
            self._p = False
            if self._name and self._url:
                self.found.append((self._name, self._url))
                self._name = ""
                self._url = ""
